# easy-yandex-s3

Использовать S3 API Яндекс.Облака еще проще.
Хранилище называется у них там Object Storage.

Не нужно. Ссылка на описание официального API Яндекса.
https://cloud.yandex.ru/docs/storage/s3/

Не нужно. Ссылка на описание S3 API Amazon (англ.)
https://docs.aws.amazon.com/en_us/AmazonS3/latest/API/Welcome.html

## Теория O'RLY

Хранилище состоит из бакетов.
Бакет (Bucket) - это, грубо говоря, ваш жесткий диск в интернетах.
Чтобы загружать что-либо в Яндекс.Облако и хранить там свои файлы, вам нужно сделать этот самый бакет.

У бакета есть ID. ID совпадает с его названием, которое вы придумаете, например **my-storage**.

Доступ к бакету можно получить с помощью специального аккаунта, его называют **сервисный аккаунт**.

От него есть логин и пароль, только логин и пароль называется **ключем**, а логин - это **"Идентификатор ключа"**, а пароль - это **"Секретный ключ"**. Не бойтесь, здесь все очень легко.

И так, предположим вы уже сделали свой бакет (как его делать, я не рассказываю, там всё довольно просто). А может даже уже сделали папки внутри (кстати, это не обязательно).

Время получить доступ для работ с бакетом!

## Создайте сервисный аккаунт в Яндекс.Облаке

1. Зайдите в ЛК
https://console.cloud.yandex.ru/cloud

2. В каталогах выберите нужный каталог в списке справа.

3. В меню слева нажмите *Сервисные аккаунты*

4. Справа сверху нажмите *Создать аккаунт*

5. Придумайте название аккаунту, оно вам не понадобится. Добавьте роли: iam.serviceAccounts.user, editor. Сохраняйте.

6. Нажмите на сервисный аккаунт, чтобы открыть его.

7. Справа сверху нажмите "Создать новый ключ" -> "Создать клч доступа"

8. Придумайте описание. Создавайте.

9. У вас появилось окошко с 2мя ключами. Скопируйте их куда-нибудь, потому что секретный ключ выдается 1 раз в этой связке. Больше вы его не увидите.

Теперь у вас есть **Идентификатор ключа** (accessKeyId) и **Секретный ключ** (secretAccessKey), поздравляю, от души.

## Как с этим работать

Установите npm-модуль в директории вашего проекта, это очень просто.
```bash
npm i easy-yandex-s3
```

### Инициализируем работу с бакетом.
```javascript
// Подключаем модуль
var EasyYandexS3 = require("EasyYandexS3");

// Инициализация
var s3 = new EasyYandexS3({
		auth: {
			accessKeyId: "ИДЕНТИФИКАТОР_КЛЮЧА",
			secretAccessKey: "ДЛИННЫЙ_СЕКРЕТНЫЙ_КЛЮЧ",
		},
		Bucket: "НАЗВАНИЕ_БАКЕТА", // например, "my-storage",
    		debug: true // Дебаг в консоли, потом можете удалить в релизе
});
```

### Загрузка файла в бакет

.Upload(***{параметры}***, ***"/папка в бакете/"***)

Загрузка по расположению файла
```javascript
  var upload = await s3.Upload({path: path.resolve(__dirname, "./123.png")}, "/test/");
  console.log(upload);  // <- Возвращает путь к файлу в хранилище и всякую дополнительную информацию.
                        // если вернулся false - произошла ошибка
                        
                        // Файл загрузится в [my-stogare]/test/{md5_сумма}.{расширение}
```

Загрузка по расположению файла, с указанием оригинального имени и расширения файла
```javascript
  var upload = await s3.Upload({
                                  path: path.resolve(__dirname, "./123.png"),
                                  save_name: true
                               }, "/test/");
  console.log(upload);  // <- Возвращает путь к файлу в хранилище и всякую дополнительную информацию.
                        // если вернулся false - произошла ошибка
                        
                        // Файл загрузится в [my-stogare]/test/123.png
```

Загрузка по расположению файла, с указанием имени файла для загрузки
```javascript
  var upload = await s3.Upload({
                                  path: path.resolve(__dirname, "./123.png"),
                                  name: "lolkek.png"
                               }, "/test/");
  console.log(upload);  // <- Возвращает путь к файлу в хранилище и всякую дополнительную информацию.
                        // если вернулся false - произошла ошибка
                        
                        // Файл загрузится в [my-stogare]/test/lolkek.png
```


Загрузка буфера
```javascript
  var upload = await s3.Upload({
                                  buffer: file_buffer
                               }, "/test/");
  console.log(upload);  // <- Возвращает путь к файлу в хранилище и всякую дополнительную информацию.
                        // если вернулся false - произошла ошибка
                        
                        // Файл загрузится в [my-stogare]/test/{md5_сумма}.{расширение}
```

Загрузка буфера с определением имени и расширения файла
```javascript
  var upload = await s3.Upload({
                                  buffer: file_buffer,
                                  name: "lolkek.png"
                               }, "/test/");
  console.log(upload);  // <- Возвращает путь к файлу в хранилище и всякую дополнительную информацию.
                        // если вернулся false - произошла ошибка
                        
                        // Файл загрузится в [my-stogare]/test/lolkek.png
```

### Получение списка директорий и файлов бакета

.GetList(***"/директория бакета/"***)

Получение корня бакета
```javascript
  var list = await s3.GetList();
```

Получение списка директорий и файлов из конкретной директории
```javascript
  var list = await s3.GetList("/test/");
```

### Скачивание файла из бакета

.Download(***"/путь до файла в бакете/"***, ***"/путь куда сохраняем на клиенте/"***)

Скачивание файла и возвращение буфера этого файла
```javascript
  var download = await s3.Download('test/123.png');
```


Скачивание файла и сохранение его в файл 
```javascript
  var download = await s3.Download('test/123.png', './myfile.png');
  
  // в download так же дополнительно вернется Buffer
  // а полученный файл с бакета будет сохранен как myfile.png в директории выполнения скрипта
```


### Удаление файла из бакета

.Remove(***"/путь до файла в бакете/"***)

Удаляем файл
```javascript
  var remove = await s3.Remove('test/123.png');

  // возвращается true или false.
  // true при успешном удалении, даже если файла нет
  // false при других критических ошибках
```
